#!/usr/bin/env ruby

['sinatra', 'sqlite3', 'set'].each do |mod|
  begin
    require mod
  rescue LoadError
    puts "ERROR: unable to require '#{mod}', is it installed? You may want to"
    puts "       try 'gem install #{mod}'"
    exit 1
  end
end

module Database

  def Database.init(db)
    fnames = %w(James John Robert Michael William David Richard Joseph Charles Thomas
              Mary Patricia Jennifer Elizabeth Linda Barbara Susan Margaret Jessica Dorothy)
    
    lnames = %w(Smith Johnson Williams Brown Jones Miller Davis Garcia Rodriguez Wilson
              Martinez Anderson Taylor Thomas Hernandez Moore Martin Jackson Thompson White Lopez
              Lee Gonzalez Harris Clark Lewis Robinson Walker Perez Hall )
    
    puts "Initializing database"
    
    db.execute <<SQL
      create table cards (
        id integer primary key autoincrement,
        fname varchar(255),
        lname varchar(255),
        pan varchar(255),
        cvv varchar(255)
      )
SQL

    bin = "4013"
    names = Set.new
    100.times do 
      digits = bin + ([(0..9).to_a]*11).map {|i| i[rand(10)]}.join
      pan = "#{digits}#{luhngen(digits)}"
      name = [fnames[rand(fnames.length)], lnames[rand(lnames.length)]]
      names << name
      cvv = sprintf("%03d", rand(1000))
      db.execute("insert into cards (fname, lname, pan, cvv) values (?, ?, ?, ?)",
                 [name[0], name[1], pan, cvv])
    end
  end

  def Database.luhngen(num)
    digits = num.scan(/./).map(&:to_i)

    sum = digits.reverse.each_slice(2).map do |x, y|
      [(x * 2).divmod(10), y || 0]
    end.flatten.inject(:+)
    
    return (10 - sum % 10) % 10
  end
end

$db = SQLite3::Database.new(':memory:')
Database::init($db)

$css    = "" +
  "body { font-family: sans-serif; color: #222; width: 80%; margin-left: auto; margin-right: auto; }\n" +
  "th { font-weight: bold; }\n"
$head   = "<style>#{$css}</style>"
$header = "<body><html><head>#{$head}</head><div><a href='/'>Home</a>&nbsp;" + 
  "|&nbsp;<a href='/list'>List</a>&nbsp;" +
  "|&nbsp;<a href='/search'>Search</a>&nbsp;" +
  "</div><h1>Simulated CDE</h1>"
$footer = "</body></html>"

use Rack::Auth::Basic, "'admin's can 'admin' here" do |username, password|
  username == 'admin' and password == 'admin'
end

def list_customers(dbrows)
  content = ""
  if dbrows.length > 0
    content += "<table><tr><th>Customer #</th><th>Name</th><th>Surname</th><th>Card Number</th></tr>"
    dbrows.each do |row|
      mask = row[3][0..3] + ("*" * 8) + row[3][-4..-1]
      memnum = sprintf("%06d", row[0].to_i)
      content += "<tr><td>#{memnum}</td><td>#{row[1]}</td><td>#{row[2]}</td><td>#{mask}</td></tr>"
    end
  else
    content += "<h2>ERROR: No entries matching query</h2>"
  end
  return content + "</table>"
end

get '/' do
  content = <<PAGE
<p>Welcome to this simulated cardholder data application.  This system stores a database of customer
records, including credit card numbers.  You can play the role of a customer support rep who can 
lookup users in the database for informational purposes.</p>
<ul>
<li><a href='/list'>List Customers</a></li>
<li><a href='/search'>Search Customers</a></li>
</ul>
PAGE
  $header + content + $footer
end

get '/list' do
  rows    = $db.execute("select id, fname, lname, pan from cards")
  content = list_customers(rows)
  $header + content + $footer
end

get '/search' do
  content = <<PAGE
<p>Use the following forms to search the customer list.</p>
<table>
<tr><td><b>Credit Card #</b></td><td><input id='paninput' name='pan' /></td><td><input type='submit' value='Search' onClick='document.location="/search/card/"+document.getElementById("paninput").value' /></td></tr>
<tr><td><b>First Name</b></td><td><input id='fnameinput' name='pan' /></td><td><input type='submit' value='Search' onClick='document.location="/search/fname/"+document.getElementById("fnameinput").value' /></td></tr>
<tr><td><b>Last Name</b></td><td><input id='lnameinput' name='pan' /></td><td><input type='submit' value='Search' onClick='document.location="/search/lname/"+document.getElementById("lnameinput").value' /></td></tr>
</table>
PAGE
  $header + content + $footer
end

get '/search/card/:pan' do
  rows = $db.execute("select id, fname, lname, pan from cards where pan like ?", [params[:pan]])
  content = list_customers(rows)
  $header + content + $footer
end

get '/search/fname/:fname' do
  rows = $db.execute("select id, fname, lname, pan from cards where fname like ?", [params[:fname]])
  content = list_customers(rows)
  $header + content + $footer
end

get '/search/lname/:lname' do
  rows = $db.execute("select id, fname, lname, pan from cards where lname like ?", [params[:lname]])
  content = list_customers(rows)
  $header + content + $footer
end

get '/customer/:num' do
  names = ["Customer #", "First Name", "Last Name", "Credit Card #", "CVV"]
  fields = $db.execute("select * id, fname, lname, pan, cvv from cards where id = ?", [params[:num].to_i])
  if fields
    fields = fields[0]
    mask = fields[3][0..3] + ("*" * 8) + fields[3][-4..-1]
    contents = <<PAGE
<table>
<tr><th>Field</th><th>Value</th></tr>
<tr><td>Customer #</td>   <td>#{fields[0]}</td></tr>
<tr><td>First Name</td>   <td>#{fields[1]}</td></tr>
<tr><td>Last Name</td>    <td>#{fields[2]}</td></tr>
<tr><td>Credit Card #</td><td>#{mask}</td></tr>
<tr><td>CVV</td>          <td>#{fields[4]}</td></tr>
</table>
PAGE
  else
    $header + "<h2>ERROR: No entries matching query</h2>" + $footer
  end
end
